@@ -3,8 +3,6 @@ use strict;
 
 package Jifty::Response;
 
-use base qw/Jifty::Object Class::Accessor/;
-
 =head1 NAME
 
 Jifty::Response - Canonical internal representation of the result of a L<Jifty::Action>
@@ -17,6 +15,10 @@ L<Jifty::Result> objects of each L<Jifty::Action> that ran.
 
 =cut
 
+use base qw/Jifty::Object Class::Accessor::Fast/;
+
+__PACKAGE__->mk_accessors(qw(error));
+
 =head2 new
 
 Creates a new L<Jifty::Response> object.
@@ -38,10 +40,10 @@ Add an HTTP header to the outgoing HTTP response.
 
 sub add_header {
     my $self = shift;
-    # This one is so we can get jifty's headers into mason
-    # Otherwise we'd have to wrap mason's output layer
      Jifty->handler->apache->header_out( @_ );
 
+    # This one is so we can get jifty's headers into mason
+    # Otherwise we'd have to wrap mason's output layer
 
     push @{$self->{headers}}, [@_];
 }
@@ -96,21 +98,28 @@ sub messages {
     return map {$_, $results{$_}->message} grep {defined $results{$_}->message and length $results{$_}->message} keys %results;
 }
 
+=head2 error [MESSAGE]
+
+Gets or sets a generalized error response.  Setting an error also
+makes the response a L</failure>.
+
 =head2 success
 
-Returns true if none of the results are failures.
+Returns true if none of the results are failures and there is no
+L</error> set.
 
 =cut
 
 sub success {
     my $self = shift;
-    return 1 unless grep {$_->failure} values %{$self->{results}};
-    return 0;
+    return 0 if grep {$_->failure} values %{$self->{results}};
+    return 1;
 }
 
 =head2 failure
 
-Returns true if any of the results failed.
+Returns true if any of the results failed or there was an L</error>
+set.
 
 =cut
 
