@@ -13,13 +13,15 @@ my $Test = Test::Builder->new;
 
 =head1 NAME
 
-Jifty::Test::WWW::Mechanize - Subclass of L<Test::WWW::Mechanize> with extra Jifty features
+Jifty::Test::WWW::Mechanize - Subclass of L<Test::WWW::Mechanize> with
+extra Jifty features
 
 =head1 METHODS
 
 =head2 new
 
-Overrides L<Test::WWW::Mechanize>'s C<new> to automatically give the bot a cookie jar.
+Overrides L<Test::WWW::Mechanize>'s C<new> to automatically give the
+bot a cookie jar.
 
 =cut
 
@@ -32,14 +34,14 @@ sub new {
 
 =head2 moniker_for ACTION, FIELD1 => VALUE1, FIELD2 => VALUE2
 
-Finds the moniker of the first action of type I<ACTION> whose "constructor"
-field I<FIELD1> is I<VALUE1>, and so on.
+Finds the moniker of the first action of type I<ACTION> whose
+"constructor" field I<FIELD1> is I<VALUE1>, and so on.
 
 =cut
 
 sub moniker_for {
   my $self = shift;
-  my $action = shift;
+  my $action = Jifty->api->qualify(shift);
   my %args = @_;
 
   for my $f ($self->forms) {
@@ -63,9 +65,10 @@ sub moniker_for {
 
 =head2 fill_in_action MONIKER, FIELD1 => VALUE1, FIELD2 => VALUE2, ...
 
-Finds the fields on the current page with the names FIELD1, FIELD2, etc in the MONIKER
-action, and fills them in.  Returns the L<HTML::Form> object of the form
-that the action is in, or undef if it can't find all the fields.
+Finds the fields on the current page with the names FIELD1, FIELD2,
+etc in the MONIKER action, and fills them in.  Returns the
+L<HTML::Form> object of the form that the action is in, or undef if it
+can't find all the fields.
 
 =cut
 
@@ -74,11 +77,8 @@ sub fill_in_action {
     my $moniker = shift;
     my %args = @_;
 
-    my $action_form = $self->action_form($moniker);
-    
-    unless ($action_form) {
-        return;
-    } 
+    my $action_form = $self->action_form($moniker, keys %args);
+    return unless $action_form;
 
     for my $arg (keys %args) {
         my $input = $action_form->find_input("J:A:F-$arg-$moniker");
@@ -93,11 +93,13 @@ sub fill_in_action {
 
 =head2 fill_in_action_ok MONIKER, FIELD1 => VALUE1, FIELD2 => VALUE2, ...
 
-Finds the fields on the current page with the names FIELD1, FIELD2, etc in the MONIKER
-action, and fills them in.  Returns the L<HTML::Form> object of the form
-that the action is in, or undef if it can't find all the fields.
+Finds the fields on the current page with the names FIELD1, FIELD2,
+etc in the MONIKER action, and fills them in.  Returns the
+L<HTML::Form> object of the form that the action is in, or undef if it
+can't find all the fields.
 
-Also, passes if it finds all of the fields and fails if any of the fields are missing.
+Also, passes if it finds all of the fields and fails if any of the
+fields are missing.
 
 =cut
 
@@ -109,33 +111,37 @@ sub fill_in_action_ok {
     $Test->ok($ret, "Filled in action $moniker");
 } 
 
-=head2 action_form MONIKER
+=head2 action_form MONIKER [ARGUMENTNAMES]
 
-Returns the form (as an L<HTML::Form> object) corresponding to the given moniker, and
-also selects it as the current form.  Returns undef if it can't be found.
+Returns the form (as an L<HTML::Form> object) corresponding to the
+given moniker (which also contains inputs for the given
+argumentnames), and also selects it as the current form.  Returns
+undef if it can't be found.
 
 =cut
 
 sub action_form {
     my $self = shift;
     my $moniker = shift;
+    my @fields = @_;
     Carp::confess("No moniker") unless $moniker;
 
     my $i;
     for my $form ($self->forms) {
         $i++;
-        if ($form->find_input("J:A-$moniker", "hidden")) {
-            $self->form_number($i); #select it, for $mech->submit etc
-            return $form;
-        } 
+        next unless $form->find_input("J:A-$moniker", "hidden");
+        next if grep {not $form->find_input("J:A:F-$_-$moniker")} @fields;
+
+        $self->form_number($i); #select it, for $mech->submit etc
+        return $form;
     } 
     return;
 } 
 
 =head2 action_field_value MONIKER, FIELD
 
-Finds the fields on the current page with the names FIELD in the action 
-MONIKER, and returns its value, or undef if it can't be found.
+Finds the fields on the current page with the names FIELD in the
+action MONIKER, and returns its value, or undef if it can't be found.
 
 =cut
 
@@ -144,7 +150,7 @@ sub action_field_value {
     my $moniker = shift;
     my $field = shift;
 
-    my $action_form = $self->action_form($moniker);
+    my $action_form = $self->action_form($moniker, $field);
     return unless $action_form;
     
     my $input = $action_form->find_input("J:A:F-$field-$moniker");
@@ -152,23 +158,16 @@ sub action_field_value {
     return $input->value;
 }
 
-=head2 field_error_text MONIKER, FIELD
-
-Finds the error span on the current page for the name FIELD in the action
-MONIKER, and returns the text (tags stripped) from it.  (If the field can't be
-found.
-
-=cut
-
 # When it sees something like
 # http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd as a DOCTYPE, this will make
-# it open dtd/xhtml1-strict.dtd instead -- great for offline hacking!
+# it open static/dtd/xhtml1-strict.dtd instead -- great for offline hacking!
  
 # This "require" is just to give us something to hook on to, and to prevent a
 # future require from taking effect.
 require 'XML/Parser/LWPExternEnt.pl';
 wrap 'XML::Parser::lwp_ext_ent_handler', pre => sub {
-    $_[2] =~ s{ \A .+ / ([^/]+) \z }{dtd/$1}xms;
+    my $root = Jifty::Util->share_root;
+    $_[2] =~ s{ \A .+ / ([^/]+) \z }{$root/dtd/$1}xms;
     open my $fh, '<', $_[2] or die "can't open $_[2]: $!";
     my $content = do {local $/; <$fh>};
     close $fh;
@@ -178,6 +177,14 @@ wrap 'XML::Parser::lwp_ext_ent_cleanup', pre => sub {
     $_[-1] = 1; # just return please
 };
 
+=head2 field_error_text MONIKER, FIELD
+
+Finds the error span on the current page for the name FIELD in the
+action MONIKER, and returns the text (tags stripped) from it.  (If the
+field can't be found.
+
+=cut
+
 sub field_error_text {
     my $self = shift;
     my $moniker = shift;
@@ -197,8 +204,9 @@ sub field_error_text {
 
 =head2 uri
 
-L<WWW::Mechanize> has a bug where it returns the wrong value for C<uri> after
-redirect.  This fixes that.  See http://rt.cpan.org/NoAuth/Bug.html?id=9059
+L<WWW::Mechanize> has a bug where it returns the wrong value for
+C<uri> after redirect.  This fixes that.  See
+http://rt.cpan.org/NoAuth/Bug.html?id=9059
 
 =cut
 
@@ -206,7 +214,8 @@ sub uri { shift->response->request->uri }
 
 =head2 get_html_ok URL
 
-Calls C<get> URL, followed by testing the HTML using L<Test::HTML::Lint>.
+Calls C<get> URL, followed by testing the HTML using
+L<Test::HTML::Lint>.
 
 =cut
 
@@ -220,7 +229,8 @@ sub get_html_ok {
 
 =head2 submit_html_ok 
 
-Calls C<submit>, followed by testing the HTML using L<Test::HTML::Lint>.
+Calls C<submit>, followed by testing the HTML using
+L<Test::HTML::Lint>.
 
 =cut
 
@@ -267,7 +277,6 @@ sub session {
 
     my $session = Jifty::Web::Session->new;
     $session->load($1);
-    $session->_session->release_all_locks();
     return $session;
 }
 
