@@ -42,4 +42,16 @@ Never render a value for a password
 sub render_value {
     return '-';
 }
+
+=head2 classes
+
+Output password fields with the class 'password'
+
+=cut
+
+sub classes {
+    my $self = shift;
+    return join(' ', 'password', ($self->SUPER::classes));
+}
+
 1;
