@@ -18,16 +18,22 @@ BEGIN {
     local $@;
     no strict 'refs';
     no warnings 'once';
-    if (eval { require YAML::Syck; $YAML::Syck::VERSION >= 0.27 }) {
-        *Load = *YAML::Syck::Load;
-        *Dump = *YAML::Syck::Dump;
+
+    if ( eval { require YAML::Syck; $YAML::Syck->VERSION(0.27) } ) {
+        *Load     = *YAML::Syck::Load;
+
+        # XXX Force non-Syck Dump until it can handle dumping circular
+        # references, which show up in halos while dumping component
+        # arguments
+        require YAML;
+        *Dump     = *YAML::Dump;
+
         *LoadFile = *YAML::Syck::LoadFile;
         *DumpFile = *YAML::Syck::DumpFile;
-    }
-    else {
+    } else {
         require YAML;
-        *Load = *YAML::Load;
-        *Dump = *YAML::Dump;
+        *Load     = *YAML::Load;
+        *Dump     = *YAML::Dump;
         *LoadFile = *YAML::LoadFile;
         *DumpFile = *YAML::DumpFile;
     }
